import json
import unittest

from nose2.tools import params
from pyramid.response import Response

from .common import create_webapp


class ResponseTests(unittest.TestCase):
    def setUp(self):
        def test(request):
            return Response(json.dumps(request.json_body),
                            content_type='application/json; charset=utf8')

        def empty(request):
            return Response(status=204)

        def plaintext(request):
            return Response('hello world', status=200,
                            content_type='text/plain; charset=utf8')

        def setup(config):
            entries = [
                ('/test_simple', test),
                ('/test_default', test),
                ('/test_not_json', plaintext),
                ('/test_empty', empty),
                ('/test_empty2', plaintext),
                ('/test_invalid_status_code', empty),
            ]
            for i, (path, handler) in enumerate(entries):
                name = 'test{}'.format(i)
                config.add_route(name, path)
                config.add_view(handler, route_name=name)

        self.app = create_webapp('test_response', [], func=setup, settings={
            'pyramid_oas3.validate_response': True
        })

    def _post(self, url, body, **kwargs):
        self.app.post_json(url, params=body, **kwargs)

    @params('simple', 'default')
    def test_simple(self, typ):
        path = '/test_{}'.format(typ)
        self._post(path, {'num': 123, 'str': 'hoge'}, status=200)
        self._post(path, {'num': 123, 'str': 'test'}, status=500)
        self._post(path, {'num': 'test'}, status=500)

    def test_not_json(self):
        self.app.get('/test_not_json', status=200)

    def test_empty(self):
        self.app.get('/test_empty', status=204)
        self.app.get('/test_empty2', status=500)

    def test_invalid_status_code(self):
        self.app.get('/test_invalid_status_code', status=500)



from base64 import b64encode
import os
import pickle

from pyramid.config import Configurator
from pyramid.response import Response
from pyramid.view import exception_view_config
from webtest import TestApp

from pyramid_oas3 import ValidationErrors, ResponseValidationError


def create_webapp(schema_name, patterns, func=None, settings=None):
    import yaml
    temp = settings
    with open(os.path.join(
            os.path.dirname(__file__), '{}.yaml'.format(schema_name))) as f:
        yaml_text = f.read()
    settings = {
        'pyramid.includes': 'pyramid_oas3',
        'pyramid_oas3.schema': yaml.load(yaml_text),
    }
    if temp:
        settings.update(temp)

    def test(request):
        return Response(b64encode(pickle.dumps(
            (request.oas3_data, request.oas3_body))))

    with Configurator(settings=settings) as config:
        for i, p in enumerate(patterns):
            name = 'test{}'.format(i)
            config.add_route(name, p)
            config.add_view(test, route_name=name)
        if func:
            func(config)
        config.scan()
        return TestApp(config.make_wsgi_app())


_last_exception = None


def get_last_exception():
    return _last_exception


@exception_view_config(ValidationErrors)
def failed_request_validation(exc, request):
    global _last_exception
    _last_exception = exc
    res = Response(str(exc))
    res.status_int = 400
    return res


@exception_view_config(ResponseValidationError)
def failed_response_validation(exc, request):
    global _last_exception
    _last_exception = exc
    res = Response(str(exc))
    res.status_int = 500
    return res



from base64 import b64decode
from datetime import date, datetime, timezone
import pickle
import unittest

from pyramid_oas3 import ValidationErrors
from .common import create_webapp, get_last_exception


class BodyTests(unittest.TestCase):
    def setUp(self):
        self.app = create_webapp(
            'test_body', [
                '/test_simple',
                '/test_fill_default',
                '/test_fill_default_oneOf',
                '/test_binary',
                '/test_fill_ref',
                '/test_fill_dict_ref',
                '/test_oneOf_error',
                '/test_not0',
                '/test_not1',
            ], settings={'pyramid_oas3.fill_by_default': True})

    def _post(self, url, body, **kwargs):
        ret = self.app.post_json(url, params=body, **kwargs)
        if ret.status_code // 100 == 2:
            return pickle.loads(b64decode(ret.body))[1]

    def test_simple(self):
        now = datetime.now(timezone.utc)
        m = {'foo': 'bar', 'hoge': 123, 'created': now.isoformat()}
        m2 = dict(m)
        m2['created'] = now
        self.assertEqual(m2, self._post('/test_simple', m, status=200))
        self._post('/test_simple', {'foo': 'bar'}, status=400)
        self._post('/test_simple', {'foo': 'hoge', 'hoge': 123}, status=400)
        self._post('/test_simple', {'foo': 'bar', 'hoge': 'piyo'}, status=400)
        self._post('/test_simple', {
            'foo': 'bar', 'hoge': 123,
            'created': '2017-12-21 00:00:00+00:00'}, status=400)
        self.app.post('/test_simple', status=406)
        self.app.post(
            '/test_simple', content_type='application/json', status=400)

    def test_fill_default(self):
        m = {'foo': 'bar', 'hoge': 123, 'created': date(2017, 7, 26)}
        m2 = {'foo': 'bar', 'hoge': 123, 'created': date(2017, 7, 26),
              'nested': {'hoge': 'piyo', 'foo': 3.14}}
        self.assertEqual(m, self._post('/test_fill_default', {}, status=200))
        self.assertEqual(m2, self._post(
            '/test_fill_default', {'nested': {}}, status=200))

    def test_fill_default_oneOf(self):
        m0, m1 = {'hoge': 'x'}, {'hoge': 'x', 'created': date(2017, 7, 26)}
        m2, m3 = {'foo': 123}, {'foo': 123, 'bar': 3.14}
        self.assertEqual(m1, self._post(
            '/test_fill_default_oneOf', m0, status=200))
        self.assertEqual(m3, self._post(
            '/test_fill_default_oneOf', m2, status=200))

    def test_binary(self):
        self._post('/test_binary', {'foo': 'bar'}, status=406)
        self.app.post('/test_binary', upload_files=[
            ('hoge', 'dummy', b'hello world')
        ])

    def test_fill_ref(self):
        m0, m1 = {}, {
            'foo': 'bar', 'hoge': 'hoge-default-value',
            'hoge2': 'hoge2-value'}
        self.assertEqual(m1, self._post(
            '/test_fill_ref', m0, status=200))

    def test_fill_dict_ref(self):
        tmp = {'foo': 'bar', 'hoge': 'hoge-default-value'}
        m0, m1 = {'required': {}}, {'required': tmp, 'filled': tmp}
        self.assertEqual(m1, self._post(
            '/test_fill_dict_ref', m0, status=200))

    def test_oneOf_error(self):
        self._post('/test_oneOf_error', 123, status=400)
        exc = get_last_exception()
        self.assertIsInstance(exc, ValidationErrors)
        self.assertIsInstance(exc.errors, list)
        self.assertEqual(len(exc.errors), 1)
        messages = sorted([str(e) for e in exc.errors[0].context])
        self.assertIn('not of type array', messages[0])
        self.assertIn('not of type object', messages[1])

    def test_not(self):
        self._post('/test_not0', 123)
        self._post('/test_not0', 'hello', status=400)
        self._post('/test_not1', 'ok')
        self._post('/test_not1', 'hello', status=400)
        self._post('/test_not1', 123, status=400)
        self._post('/test_not1', {}, status=400)



import unittest
from nose2.tools import params

from .common import create_webapp


def _setup_auth(config):
    from pyramid.authorization import ACLAuthorizationPolicy
    config.set_authorization_policy(ACLAuthorizationPolicy())
    config.set_authentication_policy(DummyAuth(['hoge']))


class SecurityTests(unittest.TestCase):
    @params('test_security_1', 'test_security_2')
    def test_auth(self, schema_name):
        app = create_webapp(
            schema_name, [
                '/',
                '/auth_required',
            ], _setup_auth)
        app.get('/', status=200)
        app.get('/auth_required', status=401)
        app.get('/auth_required', headers={'X-Auth-Token': 'hoge'}, status=200)
        app.get('/auth_required', headers={'X-Auth-Token': 'foo'}, status=401)


class DummyAuth(object):
    def __init__(self, users, header_name='X-Auth-Token'):
        self._header_name = header_name
        self._users = set(users)

    def unauthenticated_userid(self, request):
        return request.headers.get(self._header_name)

    def authenticated_userid(self, request):
        uid = self.unauthenticated_userid(request)
        if uid in self._users:
            return uid
        return None
